@@ -27,6 +27,7 @@
 #include <cpu_vcpu_cp15.h>
 #include <cpu_vcpu_helper.h>
 #include <cpu_defines.h>
+#include <arm_features.h>
 
 u32 arch_vcpu_irq_count(struct vmm_vcpu * vcpu)
 {
@@ -68,6 +69,12 @@ u32 arch_vcpu_irq_priority(struct vmm_vcpu * vcpu, u32 irq_no)
 	return ret;
 }
 
+int arch_vcpu_irq_assert(struct vmm_vcpu * vcpu, u32 irq_no, u32 reason)
+{
+	/* We don't implement this. */
+	return VMM_OK;
+}
+
 int arch_vcpu_irq_execute(struct vmm_vcpu * vcpu,
 			 arch_regs_t * regs, 
 			 u32 irq_no, u32 reason)
@@ -158,3 +165,10 @@ int arch_vcpu_irq_execute(struct vmm_vcpu * vcpu,
 
 	return VMM_OK;
 }
+
+int arch_vcpu_irq_deassert(struct vmm_vcpu * vcpu, u32 irq_no, u32 reason)
+{
+	/* We don't implement this. */
+	return VMM_OK;
+}
+
