@@ -480,105 +480,6 @@
 #define VTCR_T0SZ_MASK					0x00000003
 #define VTCR_T0SZ_SHIFT					0
 
-/* TTBL Generic */
-#define TTBL_INITIAL_TABLE_COUNT			8
-#define TTBL_TABLE_SIZE					0x00001000
-#define TTBL_TABLE_SIZE_SHIFT				12
-#define TTBL_TABLE_ENTCNT				512
-#define TTBL_TABLE_ENTSZ				8
-#define TTBL_STAGE1					1
-#define TTBL_STAGE2					2
-#define TTBL_FIRST_LEVEL				1
-#define TTBL_LEVEL1					1
-#define TTBL_LEVEL2					2
-#define TTBL_LEVEL3					3
-#define TTBL_LAST_LEVEL					3
-/* L1 index Bit[39:30] */
-#define TTBL_L1_INDEX_MASK				0x000000FFC0000000ULL
-#define TTBL_L1_INDEX_SHIFT				30
-#define TTBL_L1_BLOCK_SIZE				0x0000000040000000ULL
-#define TTBL_L1_MAP_MASK				(~(TTBL_L1_BLOCK_SIZE - 1))
-/* L2 index Bit[29:21] */
-#define TTBL_L2_INDEX_MASK				0x000000003FE00000ULL
-#define TTBL_L2_INDEX_SHIFT				21
-#define TTBL_L2_BLOCK_SIZE				0x0000000000200000ULL
-#define TTBL_L2_MAP_MASK				(~(TTBL_L2_BLOCK_SIZE - 1))
-/* L3 index Bit[20:12] */
-#define TTBL_L3_INDEX_MASK				0x00000000001FF000ULL
-#define TTBL_L3_INDEX_SHIFT				12
-#define TTBL_L3_BLOCK_SIZE				0x0000000000001000ULL
-#define TTBL_L3_MAP_MASK				(~(TTBL_L3_BLOCK_SIZE - 1))
-#define TTBL_UPPER_MASK					0xFFF0000000000000ULL
-#define TTBL_UPPER_SHIFT				52
-#define TTBL_OUTADDR_MASK				0x000000FFFFFFF000ULL
-#define TTBL_OUTADDR_SHIFT				12
-#define TTBL_AP_SRW_U					0x0
-#define TTBL_AP_S_URW					0x1
-#define TTBL_AP_SR_U					0x2
-#define TTBL_AP_S_UR					0x3
-#define TTBL_HAP_NOACCESS				0x0
-#define TTBL_HAP_READONLY				0x1
-#define TTBL_HAP_WRITEONLY				0x2
-#define TTBL_HAP_READWRITE				0x3
-#define TTBL_LOWER_MASK					0x0000000000000FFCULL
-#define TTBL_LOWER_SHIFT				2
-#define TTBL_TABLE_MASK					0x0000000000000002ULL
-#define TTBL_TABLE_SHIFT				1
-#define TTBL_VALID_MASK					0x0000000000000001ULL
-#define TTBL_VALID_SHIFT				0
-
-/* TTBL Stage1 Table Attributes */
-#define TTBL_STAGE1_TABLE_NS_MASK			0x8000000000000000ULL
-#define TTBL_STAGE1_TABLE_NS_SHIFT			63
-#define TTBL_STAGE1_TABLE_AP_MASK			0x6000000000000000ULL
-#define TTBL_STAGE1_TABLE_AP_SHIFT			61
-#define TTBL_STAGE1_TABLE_XN_MASK			0x1000000000000000ULL
-#define TTBL_STAGE1_TABLE_XN_SHIFT			60
-#define TTBL_STAGE1_TABLE_PXN_MASK			0x0800000000000000ULL
-#define TTBL_STAGE1_TABLE_PXN_SHIFT			59
-
-/* TTBL Stage1 Block Upper Attributes */
-#define TTBL_STAGE1_UPPER_XN_MASK			0x0040000000000000ULL
-#define TTBL_STAGE1_UPPER_XN_SHIFT			54
-#define TTBL_STAGE1_UPPER_PXN_MASK			0x0020000000000000ULL
-#define TTBL_STAGE1_UPPER_PXN_SHIFT			53
-#define TTBL_STAGE1_UPPER_CONT_MASK			0x0010000000000000ULL
-#define TTBL_STAGE1_UPPER_CONT_SHIFT			52
-
-/* TTBL Stage1 Block Lower Attributes */
-#define TTBL_STAGE1_LOWER_NG_MASK			0x0000000000000800ULL
-#define TTBL_STAGE1_LOWER_NG_SHIFT			11
-#define TTBL_STAGE1_LOWER_AF_MASK			0x0000000000000400ULL
-#define TTBL_STAGE1_LOWER_AF_SHIFT			10
-#define TTBL_STAGE1_LOWER_SH_MASK			0x0000000000000300ULL
-#define TTBL_STAGE1_LOWER_SH_SHIFT			8
-#define TTBL_STAGE1_LOWER_AP_MASK			0x00000000000000C0ULL
-#define TTBL_STAGE1_LOWER_AP_SHIFT			6
-#define TTBL_STAGE1_LOWER_NS_MASK			0x0000000000000020ULL
-#define TTBL_STAGE1_LOWER_NS_SHIFT			5
-#define TTBL_STAGE1_LOWER_AINDEX_MASK			0x000000000000001CULL
-#define TTBL_STAGE1_LOWER_AINDEX_SHIFT			2
-
-/* TTBL Stage2 Block Upper Attributes */
-#define TTBL_STAGE2_UPPER_MASK				0xFFF0000000000000ULL
-#define TTBL_STAGE2_UPPER_SHIFT				52
-#define TTBL_STAGE2_UPPER_XN_MASK			0x0040000000000000ULL
-#define TTBL_STAGE2_UPPER_XN_SHIFT			54
-#define TTBL_STAGE2_UPPER_CONT_MASK			0x0010000000000000ULL
-#define TTBL_STAGE2_UPPER_CONT_SHIFT			52
-
-/* TTBL Stage2 Block Lower Attributes */
-#define TTBL_STAGE2_LOWER_MASK				0x0000000000000FFCULL
-#define TTBL_STAGE2_LOWER_SHIFT				2
-#define TTBL_STAGE2_LOWER_AF_MASK			0x0000000000000400ULL
-#define TTBL_STAGE2_LOWER_AF_SHIFT			10
-#define TTBL_STAGE2_LOWER_SH_MASK			0x0000000000000300ULL
-#define TTBL_STAGE2_LOWER_SH_SHIFT			8
-#define TTBL_STAGE2_LOWER_HAP_MASK			0x00000000000000C0ULL
-#define TTBL_STAGE2_LOWER_HAP_SHIFT			6
-#define TTBL_STAGE2_LOWER_MEMATTR_MASK			0x000000000000003CULL
-#define TTBL_STAGE2_LOWER_MEMATTR_SHIFT			2
-
 /* HMAIR encodings */
 #define AINDEX_SO					0
 #define AINDEX_NORMAL_WT				1
