@@ -37,13 +37,15 @@
 #define VMM_DEVTREE_DEVICE_TYPE_VAL_RAM		"ram"
 #define VMM_DEVTREE_DEVICE_TYPE_VAL_ROM		"rom"
 #define VMM_DEVTREE_COMPATIBLE_ATTR_NAME	"compatible"
-#define VMM_DEVTREE_CLOCK_RATE_ATTR_NAME	"clock-rate"
+#define VMM_DEVTREE_CLOCK_FREQ_ATTR_NAME	"clock-frequency"
 #define VMM_DEVTREE_REG_ATTR_NAME		"reg"
 #define VMM_DEVTREE_VIRTUAL_REG_ATTR_NAME	"virtual-reg"
 
-#define VMM_DEVTREE_CHOOSEN_NODE_NAME		"choosen"
+#define VMM_DEVTREE_CHOSEN_NODE_NAME		"chosen"
 #define VMM_DEVTREE_CONSOLE_ATTR_NAME		"console"
 #define VMM_DEVTREE_RTCDEV_ATTR_NAME		"rtcdev"
+#define VMM_DEVTREE_BOOTARGS_ATTR_NAME		"bootargs"
+#define VMM_DEVTREE_BOOTCMD_ATTR_NAME		"bootcmd"
 
 #define VMM_DEVTREE_ALIASES_NODE_NAME		"aliases"
 
@@ -51,20 +53,22 @@
 #define VMM_DEVTREE_VMMNET_NODE_NAME		"net"
 #define VMM_DEVTREE_NETSTACK_NODE_NAME		"hoststack"
 
-#define VMM_DEVTREE_HOSTINFO_NODE_NAME		"host"
-#define VMM_DEVTREE_CPUS_NODE_NAME		"cpus"
-#define VMM_DEVTREE_CPU_FREQ_MHZ_ATTR_NAME	"cpu_freq_mhz"
 #define VMM_DEVTREE_MEMORY_NODE_NAME		"memory"
 #define VMM_DEVTREE_MEMORY_PHYS_ADDR_ATTR_NAME	"physical_addr"
 #define VMM_DEVTREE_MEMORY_PHYS_SIZE_ATTR_NAME	"physical_size"
 
+#define VMM_DEVTREE_CPUS_NODE_NAME		"cpus"
+#define VMM_DEVTREE_CPU_FREQ_MHZ_ATTR_NAME	"cpu_freq_mhz"
+#define VMM_DEVTREE_INTERRUPTS_ATTR_NAME	"interrupts"
+
 #define VMM_DEVTREE_GUESTINFO_NODE_NAME		"guests"
 #define VMM_DEVTREE_VCPUS_NODE_NAME		"vcpus"
 #define VMM_DEVTREE_START_PC_ATTR_NAME		"start_pc"
 #define VMM_DEVTREE_PRIORITY_ATTR_NAME		"priority"
 #define VMM_DEVTREE_TIME_SLICE_ATTR_NAME	"time_slice"
 #define VMM_DEVTREE_ADDRSPACE_NODE_NAME		"aspace"
-#define VMM_DEVTREE_H2GIRQMAP_ATTR_NAME		"h2g_irq_map"
+#define VMM_DEVTREE_GUESTIRQCNT_ATTR_NAME	"guest_irq_count"
+#define VMM_DEVTREE_H2GIRQMAP_ATTR_NAME		"host2guest_irq_map"
 #define VMM_DEVTREE_MANIFEST_TYPE_ATTR_NAME	"manifest_type"
 #define VMM_DEVTREE_MANIFEST_TYPE_VAL_REAL	"real"
 #define VMM_DEVTREE_MANIFEST_TYPE_VAL_VIRTUAL	"virtual"
@@ -77,6 +81,7 @@
 #define VMM_DEVTREE_ALIAS_PHYS_ATTR_NAME	"alias_physical_addr"
 #define VMM_DEVTREE_PHYS_SIZE_ATTR_NAME		"physical_size"
 #define VMM_DEVTREE_SWITCH_ATTR_NAME		"switch"
+#define VMM_DEVTREE_BLKDEV_ATTR_NAME		"blkdev"
 
 enum vmm_devtree_attrypes {
 	VMM_DEVTREE_ATTRTYPE_UNKNOWN = 0,
@@ -97,6 +102,13 @@ struct vmm_devtree_attr {
 	u32 len;
 };
 
+struct vmm_devtree_nodeid {
+	char name[32];
+	char type[32];
+	char compatible[128];
+	void *data;
+};
+
 struct vmm_devtree_node {
 	struct dlist head;
 	char *name;
@@ -120,6 +132,25 @@ u32 vmm_devtree_literal_size(u32 attrtype);
 /** Estimate type of attribute from its name */
 u32 vmm_devtree_estimate_attrtype(const char *name);
 
+/** Get device clock-frequency
+ *  Note: This is based on 'clock-frequency' attribute of device tree node
+ *  Note: This API if for hard-coding clcok frequency in device tree node 
+ *  and it does not use arch_clk_xxxx() APIs
+ */
+int vmm_devtree_clock_frequency(struct vmm_devtree_node *node, 
+				u32 *clock_freq);
+
+/** Get device irq number
+ *  Note: This is based on 'irq' attribute of device tree node
+ */
+int vmm_devtree_irq_get(struct vmm_devtree_node *node, 
+		        u32 *irq, int index);
+
+/** Get count of device irqs
+ *  Note: This is based on 'irq' attribute of device tree node
+ */
+u32 vmm_devtree_irq_count(struct vmm_devtree_node *node);
+
 /** Map device registers to virtual address
  *  Note: This is based on 'reg' and 'virtual-reg' attributes 
  *  of device tree node
@@ -148,9 +179,16 @@ int vmm_devtree_regaddr(struct vmm_devtree_node *node,
 int vmm_devtree_regunmap(struct vmm_devtree_node *node, 
 			 virtual_addr_t addr, int regset);
 
+/** Match a node with nodeid table
+ *  Returns NULL if node does not match otherwise nodeid table entry
+ */
+const struct vmm_devtree_nodeid *vmm_devtree_match_node(
+				const struct vmm_devtree_nodeid *matches,
+				struct vmm_devtree_node *node);
+
 /** Get attribute value */
-void * vmm_devtree_attrval(struct vmm_devtree_node *node, 
-			   const char *attrib);
+void *vmm_devtree_attrval(struct vmm_devtree_node *node, 
+			  const char *attrib);
 
 /** Get length of attribute value */
 u32 vmm_devtree_attrlen(struct vmm_devtree_node *node, 
@@ -173,13 +211,30 @@ int vmm_devtree_delattr(struct vmm_devtree_node *node, const char *name);
 /** Create a path string for a given node */
 int vmm_devtree_getpath(char *out, struct vmm_devtree_node *node);
 
-/** Get node corresponding to a path string */
-struct vmm_devtree_node *vmm_devtree_getnode(const char *path);
-
 /** Get child node below a given node */
 struct vmm_devtree_node *vmm_devtree_getchild(struct vmm_devtree_node *node,
 					      const char *path);
 
+/** Get node corresponding to a path string 
+ *  Note: If path == NULL then root node will be returned
+ */
+struct vmm_devtree_node *vmm_devtree_getnode(const char *path);
+
+/** Find node matching nodeid table starting from given node 
+ *  Note: If node == NULL then node == root
+ */
+struct vmm_devtree_node *vmm_devtree_find_matching(
+				struct vmm_devtree_node *node,
+				const struct vmm_devtree_nodeid *matches);
+
+/** Find compatible node starting from given node 
+ *  Note: If node == NULL then node == root 
+ */
+struct vmm_devtree_node *vmm_devtree_find_compatible(
+				struct vmm_devtree_node *node,
+				const char *device_type,
+				const char *compatible);
+
 /** Add new node to device tree */
 struct vmm_devtree_node *vmm_devtree_addnode(struct vmm_devtree_node *parent,
 					     const char *name);
@@ -192,9 +247,6 @@ int vmm_devtree_copynode(struct vmm_devtree_node *parent,
 /** Delete a node from device tree */
 int vmm_devtree_delnode(struct vmm_devtree_node *node);
 
-/** Get the root node */
-struct vmm_devtree_node *vmm_devtree_rootnode(void);
-
 /** Initialize device tree */
 int vmm_devtree_init(void);
 
