@@ -24,12 +24,12 @@
 #include <vmm_profiler.h>
 #include <vmm_heap.h>
 #include <vmm_error.h>
-#include <vmm_string.h>
 #include <vmm_timer.h>
-#include <arch_cpu.h>
 #include <vmm_stdio.h>
 #include <vmm_spinlocks.h>
-#include <kallsyms.h>
+#include <arch_cpu.h>
+#include <libs/stringlib.h>
+#include <libs/kallsyms.h>
 
 typedef void (*vmm_profile_callback_t) (void *, void *);
 
@@ -87,7 +87,7 @@ static void __notrace vmm_profile_enter(void *ip, void *parent_ip)
 		goto out;
 	}
 
-	flags = vmm_spin_lock_irqsave(&pctrl.lock);
+	vmm_spin_lock_irqsave(&pctrl.lock, flags);
 
 	pctrl.stat[index].counter++;
 	pctrl.stat[index].is_tracing = 1;
@@ -123,7 +123,7 @@ static void __notrace vmm_profile_exit(void *ip, void *parent_ip)
 		goto out;
 	}
 
-	flags = vmm_spin_lock_irqsave(&pctrl.lock);
+	vmm_spin_lock_irqsave(&pctrl.lock, flags);
 
 	time = vmm_timer_timestamp_for_profile();
 
@@ -151,11 +151,13 @@ bool vmm_profiler_isactive(void)
 int vmm_profiler_start(void)
 {
 	if (!vmm_profiler_isactive()) {
-		irq_flags_t flags = arch_cpu_irq_save();
+		irq_flags_t flags; 
 
-		vmm_memset(pctrl.stat, 0,
-			   sizeof(struct vmm_profiler_stat) *
-			   kallsyms_num_syms);
+		arch_cpu_irq_save(flags);
+
+		memset(pctrl.stat, 0,
+			sizeof(struct vmm_profiler_stat) *
+			kallsyms_num_syms);
 		_vmm_profile_enter = vmm_profile_enter;
 		_vmm_profile_exit = vmm_profile_exit;
 		pctrl.is_active = 1;
@@ -171,7 +173,9 @@ int vmm_profiler_start(void)
 int vmm_profiler_stop(void)
 {
 	if (vmm_profiler_isactive()) {
-		irq_flags_t flags = arch_cpu_irq_save();
+		irq_flags_t flags; 
+
+		arch_cpu_irq_save(flags);
 
 		_vmm_profile_enter = vmm_profile_none;
 		_vmm_profile_exit = vmm_profile_none;
@@ -204,7 +208,7 @@ int __init vmm_profiler_init(void)
 		return VMM_EFAIL;
 	}
 
-	vmm_memset(pctrl.stat, 0, sizeof(struct vmm_profiler_stat) *
+	memset(pctrl.stat, 0, sizeof(struct vmm_profiler_stat) *
 		   kallsyms_num_syms);
 
 	INIT_SPIN_LOCK(&pctrl.lock);
