@@ -65,7 +65,6 @@ struct gic_irq_state {
 
 struct gic_state {
 	struct vmm_guest *guest;
-	struct vmm_emupic *pic;
 	vmm_spinlock_t lock;
 
 	/* Configuration */
@@ -73,7 +72,7 @@ struct gic_state {
 	u8 id[8];
 	u32 num_cpu;
 	u32 num_irq;
-	u32 num_base_irq;
+	u32 base_irq;
 	bool is_child_pic;
 	u32 parent_irq[GIC_MAX_NCPU];
 
@@ -193,19 +192,16 @@ static void gic_update(struct gic_state *s)
 	vmm_spin_unlock_irqrestore(&s->lock, flags);
 }
 
-/* Process IRQ asserted in device emulation framework */
-static int gic_irq_handle(struct vmm_emupic *epic, u32 irq, int cpu, int level)
+/* Process IRQ asserted via device emulation framework */
+static void gic_irq_handle(u32 irq, int cpu, int level, void *opaque)
 {
 	irq_flags_t flags;
 	int cm, target;
-	struct gic_state *s = (struct gic_state *)epic->priv;
+	struct gic_state *s = opaque;
 
-	/* Ensure irq is in range (base_irq, base_irq + num_irq) */
-	if ((s->num_base_irq + s->num_irq) <= irq) {
-		return VMM_EMUPIC_IRQ_UNHANDLED;
-	}
+	irq -= s->base_irq;
 
-	if (irq < (s->num_base_irq + 32)) {
+	if (irq < 32) {
 		/* In case of PPIs and SGIs */
 		cm = target = (1 << cpu);
 	} else {
@@ -218,7 +214,7 @@ static int gic_irq_handle(struct vmm_emupic *epic, u32 irq, int cpu, int level)
 
 	if (level == GIC_TEST_LEVEL(s, irq, cm)) {
 		vmm_spin_unlock_irqrestore(&s->lock, flags);
-		return VMM_EMUPIC_IRQ_HANDLED;
+		return;
 	}
 
 	if (level) {
@@ -233,8 +229,6 @@ static int gic_irq_handle(struct vmm_emupic *epic, u32 irq, int cpu, int level)
 	__gic_update(s);
 
 	vmm_spin_unlock_irqrestore(&s->lock, flags);
-
-	return VMM_EMUPIC_IRQ_HANDLED;
 }
 
 static void __gic_set_running_irq(struct gic_state *s, int cpu, int irq)
@@ -321,7 +315,7 @@ static void gic_complete_irq(struct gic_state * s, int cpu, int irq)
 	vmm_spin_unlock_irqrestore(&s->lock, flags);
 
 	/* Signal completion of host-to-guest mapped irq */
-	vmm_devemu_complete_h2g_irq(s->guest, irq);
+	vmm_devemu_complete_host2guest_irq(s->guest, irq);
 }
 
 static int __gic_dist_readb(struct gic_state * s, int cpu, u32 offset, u8 *dst)
@@ -917,7 +911,7 @@ int gic_state_reset(struct gic_state *s)
 	}
 
 	for (i = 0; i < GIC_NUM_CPU(s); i++) {
-		if (s->type == GIC_TYPE_11MPCORE) {
+		if (s->type == GIC_TYPE_ARM11MPCORE) {
 			s->priority_mask[i] = 0xf0;
 		} else {
 			s->priority_mask[i] = 0x0;
@@ -950,7 +944,23 @@ static int gic_emulator_reset(struct vmm_emudev *edev)
 static u32 gic_configs[][14] = {
 	{ 
 		/* num_irq */ 96,
-		/* num_base_irq */ 0,
+		/* base_irq */ 0,
+		/* id0 */ 0x90, 
+		/* id1 */ 0x13, 
+		/* id2 */ 0x04, 
+		/* id3 */ 0x00, 
+		/* id4 */ 0x0d, 
+		/* id5 */ 0xf0, 
+		/* id6 */ 0x05, 
+		/* id7 */ 0xb1,
+		/* cpu_offset */ 0x100,
+		/* cpu_length */ 0x100,
+		/* dist_offset */ 0x1000,
+		/* dist_length */ 0x1000,
+	},
+	{ 
+		/* num_irq */ 96,
+		/* base_irq */ 0,
 		/* id0 */ 0x90, 
 		/* id1 */ 0x13, 
 		/* id2 */ 0x04, 
@@ -966,23 +976,7 @@ static u32 gic_configs[][14] = {
 	},
 	{ 
 		/* num_irq */ 96,
-		/* num_base_irq */ 0,
-		/* id0 */ 0x90, 
-		/* id1 */ 0x13, 
-		/* id2 */ 0x04, 
-		/* id3 */ 0x00, 
-		/* id4 */ 0x0d, 
-		/* id5 */ 0xf0, 
-		/* id6 */ 0x05, 
-		/* id7 */ 0xb1,
-		/* cpu_offset */ 0x0,
-		/* cpu_length */ 0x100,
-		/* dist_offset */ 0x1000,
-		/* dist_length */ 0x1000,
-	},
-	{ 
-		/* num_irq */ 96,
-		/* num_base_irq */ 0,
+		/* base_irq */ 0,
 		/* id0 */ 0x90, 
 		/* id1 */ 0x13, 
 		/* id2 */ 0x04, 
@@ -998,7 +992,7 @@ static u32 gic_configs[][14] = {
 	},
 	{ 
 		/* num_irq */ 128,
-		/* num_base_irq */ 0,
+		/* base_irq */ 0,
 		/* id0 */ 0x90, 
 		/* id1 */ 0x13, 
 		/* id2 */ 0x04, 
@@ -1014,38 +1008,26 @@ static u32 gic_configs[][14] = {
 	},
 };
 
-struct gic_state *gic_state_alloc(struct vmm_guest *guest,
-			   enum gic_type type,
-			   u32 num_cpu,
-			   bool is_child_pic,
-	       		   u32 parent_irq[])
+struct gic_state *gic_state_alloc(const char *name,
+				  struct vmm_guest *guest, 
+				  enum gic_type type,
+				  u32 num_cpu, 
+				  bool is_child_pic,
+				  u32 base_irq,
+				  u32 num_irq,
+				  u32 parent_irq)
 {
-	int i;
+	u32 i;
 	struct gic_state *s = NULL;
 
-	s = vmm_malloc(sizeof(struct gic_state));
+	s = vmm_zalloc(sizeof(struct gic_state));
 	if (!s) {
-		goto gic_emulator_init_done;
-	}
-	memset(s, 0x0, sizeof(struct gic_state));
-
-	s->pic = vmm_malloc(sizeof(struct vmm_emupic));
-	if (!s->pic) {
-		goto gic_emulator_init_freestate_failed;
-	}
-	memset(s->pic, 0x0, sizeof(struct vmm_emupic));
-
-	strcpy(s->pic->name, "gic-pic");
-	s->pic->type = VMM_EMUPIC_IRQCHIP;
-	s->pic->handle = &gic_irq_handle;
-	s->pic->priv = s;
-	if (vmm_devemu_register_pic(guest, s->pic)) {
-		goto gic_emulator_init_freepic_failed;
+		return NULL;
 	}
 
 	s->num_cpu = num_cpu;
-	s->num_irq = gic_configs[type][0];
-	s->num_base_irq = gic_configs[type][1];
+	s->num_irq = num_irq;
+	s->base_irq = base_irq;
 	s->type = type;
 	s->id[0] = gic_configs[type][2];
 	s->id[1] = gic_configs[type][3];
@@ -1063,33 +1045,29 @@ struct gic_state *gic_state_alloc(struct vmm_guest *guest,
 	s->is_child_pic = is_child_pic;
 
 	for (i = 0; i < s->num_cpu; i++) {
-		s->parent_irq[i] = parent_irq[i];
+		s->parent_irq[i] = parent_irq;
 	}
 
 	s->guest = guest;
 	INIT_SPIN_LOCK(&s->lock);
 
-	goto gic_emulator_init_done;
+	for (i = s->base_irq; i < (s->base_irq + s->num_irq); i++) {
+		vmm_devemu_register_irq_handler(guest, i, 
+						name, gic_irq_handle, s);
+	}
 
-gic_emulator_init_freepic_failed:
-	vmm_free(s->pic);
-gic_emulator_init_freestate_failed:
-	vmm_free(s);
-gic_emulator_init_done:
 	return s;
 }
 VMM_EXPORT_SYMBOL(gic_state_alloc);
 
 int gic_state_free(struct gic_state *s)
 {
-	int rc;
-	if(s) {
-		if (s->pic) {
-			rc = vmm_devemu_unregister_pic(s->guest, s->pic);
-			if (rc) {
-				return rc;
-			}
-			vmm_free(s->pic);
+	u32 i;
+
+	if (s) {
+		for (i = s->base_irq; i < (s->base_irq + s->num_irq); i++) {
+			vmm_devemu_unregister_irq_handler(s->guest, i,
+							  gic_irq_handle, s);
 		}
 		vmm_free(s);
 		return VMM_OK;
@@ -1101,15 +1079,13 @@ VMM_EXPORT_SYMBOL(gic_state_free);
 
 static int gic_emulator_probe(struct vmm_guest *guest,
 			      struct vmm_emudev *edev,
-			      const struct vmm_emuid *eid)
+			      const struct vmm_devtree_nodeid *eid)
 {
-	u32 attrlen;
 	const char *attr;
 	struct gic_state *s;
 	bool is_child_pic;
 	enum gic_type type;
-	u32 *parent_irq;
-	u32 num_cpu;
+	u32 parent_irq, base_irq, num_irq;
 
 	attr = vmm_devtree_attrval(edev->node, "child_pic");
 	if (attr) {
@@ -1122,16 +1098,27 @@ static int gic_emulator_probe(struct vmm_guest *guest,
 	if (!attr) {
 		return VMM_EFAIL;
 	}
-	attrlen = vmm_devtree_attrlen(edev->node, "parent_irq");
-	num_cpu = (attrlen / sizeof(u32));
-	if (guest->vcpu_count < num_cpu) {
-		num_cpu = guest->vcpu_count;
-	}
-	parent_irq = (u32 *)attr;
+	parent_irq = *((u32 *)attr);
 	
 	type = (enum gic_type)(eid->data);
 
-	s = gic_state_alloc(guest, type, num_cpu, is_child_pic, parent_irq);
+	attr = vmm_devtree_attrval(edev->node, "base_irq");
+	if (!attr) {
+		base_irq = gic_configs[type][1];
+	} else {
+		base_irq = *((u32 *)attr);
+	}
+	
+	attr = vmm_devtree_attrval(edev->node, "num_irq");
+	if (!attr) {
+		num_irq = gic_configs[type][0];
+	} else {
+		num_irq = *((u32 *)attr);
+	}
+	
+	s = gic_state_alloc(edev->node->name, guest, type, 
+			    guest->vcpu_count, is_child_pic,
+			    base_irq, num_irq, parent_irq);
 
 	edev->priv = s;
 
@@ -1148,7 +1135,11 @@ static int gic_emulator_remove(struct vmm_emudev *edev)
 	return VMM_OK;
 }
 
-static struct vmm_emuid gic_emuid_table[] = {
+static struct vmm_devtree_nodeid gic_emuid_table[] = {
+	{ .type = "pic", 
+	  .compatible = "arm11mpcore,gic", 
+	  .data = (void *)GIC_TYPE_ARM11MPCORE,
+	},
 	{ .type = "pic", 
 	  .compatible = "realview,gic", 
 	  .data = (void *)GIC_TYPE_REALVIEW,
