@@ -38,6 +38,14 @@
 #define	MODULE_INIT			vfs_init
 #define	MODULE_EXIT			vfs_exit
 
+/** file descriptor structure */
+struct file {
+	struct vmm_mutex f_lock;	/* file lock */
+	u32 f_flags;			/* open flag */
+	loff_t f_offset;		/* current position in file */
+	struct vnode *f_vnode;		/* vnode */
+};
+
 /* size of vnode hash table, must power 2 */
 #define VFS_VNODE_HASH_SIZE		(32)
 
@@ -51,10 +59,76 @@ struct vfs_ctrl {
 	struct vmm_mutex fd_bmap_lock;
 	unsigned long *fd_bmap;
 	struct file fd[VFS_MAX_FD];
+	struct vmm_notifier_block bdev_client;
 };
 
 static struct vfs_ctrl vfsc;
 
+/** Compare two path strings and return matched length. */
+static int count_match(const char *path, char *mount_root)
+{
+	int len = 0;
+
+	while (*path && *mount_root) {
+		if ((*path++) != (*mount_root++))
+			break;
+		len++;
+	}
+
+	if (*mount_root != '\0') {
+		return 0;
+	}
+
+	if ((len == 1) && (*(path - 1) == '/')) {
+		return 1;
+	}
+
+	if ((*path == '\0') || (*path == '/')) {
+		return len;
+	}
+
+	return 0;
+}
+
+static int vfs_findroot(const char *path, struct mount **mp, char **root)
+{
+	struct dlist *l;
+	struct mount *m, *tmp;
+	int len, max_len = 0;
+
+	if (!path || !mp || !root) {
+		return VMM_EFAIL;
+	}
+
+	/* find mount point from nearest path */
+	m = NULL;
+
+	vmm_mutex_lock(&vfsc.mnt_list_lock);
+
+	list_for_each(l, &vfsc.mnt_list) {
+		tmp = list_entry(l, struct mount, m_link);
+		len = count_match(path, tmp->m_path);
+		if (len > max_len) {
+			max_len = len;
+			m = tmp;
+		}
+	}
+
+	vmm_mutex_unlock(&vfsc.mnt_list_lock);
+
+	if (m == NULL) {
+		return VMM_EFAIL;
+	}
+
+	*root = (char *)(path + max_len);
+	while (**root == '/') {
+		(*root)++;
+	}
+	*mp = m;
+
+	return VMM_OK;
+}
+
 static int vfs_fd_alloc(void)
 {
 	int i, ret = -1;
@@ -73,34 +147,6 @@ static int vfs_fd_alloc(void)
 	return ret;
 }
 
-/* FIXME: If a mount point is removed abruptly (probably due to
- * unplugging of a pluggable block device) then we need to
- * flush all file descriptors and vnodes under this mount point.
- *
- * Not sure whether:
- * vfs_fd_umount_flush() handles all possible scenarios !!!!
- */
-static void vfs_fd_unmount_flush(struct mount *m)
-{
-	int i;
-
-	vmm_mutex_lock(&vfsc.fd_bmap_lock);
-
-	for (i = 0; i < VFS_MAX_FD; i++) {
-		if (bitmap_isset(vfsc.fd_bmap, i) &&
-		    (vfsc.fd[i].f_vnode->v_mount == m)) {
-			vmm_mutex_lock(&vfsc.fd[i].f_lock);
-			vfsc.fd[i].f_flags = 0;
-			vfsc.fd[i].f_offset = 0;
-			vfsc.fd[i].f_vnode = NULL;
-			vmm_mutex_unlock(&vfsc.fd[i].f_lock);
-			bitmap_clear(vfsc.fd_bmap, i, 1);
-		}
-	}
-
-	vmm_mutex_unlock(&vfsc.fd_bmap_lock);
-}
-
 static void vfs_fd_free(int fd)
 {
 	if (-1 < fd && fd < VFS_MAX_FD) {
@@ -152,10 +198,6 @@ static struct vnode *vfs_vnode_vget(struct mount *m, const char *path)
 	}
 
 	len = strlen(path) + 1;
-	if (!(v->v_path = vmm_malloc(len))) {
-		vmm_free(v);
-		return NULL;
-	}
 
 	INIT_LIST_HEAD(&v->v_link);
 	INIT_MUTEX(&v->v_lock);
@@ -168,7 +210,6 @@ static struct vnode *vfs_vnode_vget(struct mount *m, const char *path)
 	err = m->m_fs->vget(m, v);
 	vmm_mutex_unlock(&m->m_lock);
 	if (err) {
-		vmm_free(v->v_path);
 		vmm_free(v);
 		return NULL;
 	}
@@ -185,10 +226,10 @@ static struct vnode *vfs_vnode_vget(struct mount *m, const char *path)
 static struct vnode *vfs_vnode_lookup(struct mount *m, const char *path)
 {
 	u32 hash;
+	bool found = FALSE;
 	struct dlist *l;
-	struct vnode *v;
+	struct vnode *v = NULL;
 
-	v = NULL;
 	hash = vfs_vnode_hash(m, path);
 
 	vmm_mutex_lock(&vfsc.vnode_list_lock[hash]);
@@ -197,16 +238,19 @@ static struct vnode *vfs_vnode_lookup(struct mount *m, const char *path)
 		v = list_entry(l, struct vnode, v_link);
 		if ((v->v_mount == m) && 
 		    (!strncmp(v->v_path, path, VFS_MAX_PATH))) {
+			found = TRUE;
 			break;
 		}
 	}
 
 	vmm_mutex_unlock(&vfsc.vnode_list_lock[hash]);
 
-	if (v) {
-		arch_atomic_add(&v->v_refcnt, 1);
+	if (!found) {
+		return NULL;
 	}
 
+	arch_atomic_add(&v->v_refcnt, 1);
+
 	return v;
 }
 
@@ -236,40 +280,9 @@ static void vfs_vnode_vput(struct vnode *v)
 
 	arch_atomic_sub(&v->v_mount->m_refcnt, 1);
 
-	vmm_free(v->v_path);
 	vmm_free(v);
 }
 
-static void vfs_vnode_unmount_flush(struct mount *m)
-{
-	int i;
-	struct dlist *l;
-	struct vnode *v;
-
-	for(i = 0; i < VFS_VNODE_HASH_SIZE; i++) {
-		vmm_mutex_lock(&vfsc.vnode_list_lock[i]);
-
-		list_for_each(l, &vfsc.vnode_list[i]) {
-			v = list_entry(l, struct vnode, v_link);
-			if(v->v_mount != m) {
-				continue;
-			}
-
-			list_del(&v->v_link);
-
-			/* deallocate fs specific data from this vnode */
-			v->v_mount->m_fs->vput(v->v_mount, v);
-
-			arch_atomic_sub(&v->v_mount->m_refcnt, 1);
-
-			vmm_free(v->v_path);
-			vmm_free(v);
-		}
-
-		vmm_mutex_unlock(&vfsc.vnode_list_lock[i]);
-	}
-}
-
 /** Get stat from vnode pointer. */
 static int vfs_vnode_stat(struct vnode *v, struct stat *st)
 {
@@ -281,6 +294,9 @@ static int vfs_vnode_stat(struct vnode *v, struct stat *st)
 	vmm_mutex_lock(&v->v_lock);
 	st->st_size = v->v_size;
 	mode = v->v_mode & (S_IRWXU|S_IRWXG|S_IRWXO);
+	st->st_ctime = v->v_ctime;
+	st->st_atime = v->v_atime;
+	st->st_mtime = v->v_mtime;
 	vmm_mutex_unlock(&v->v_lock);
 
 	switch (v->v_type) {
@@ -316,10 +332,6 @@ static int vfs_vnode_stat(struct vnode *v, struct stat *st)
 	st->st_uid = 0;
 	st->st_gid = 0;
 
-	st->st_ctime = 0;
-	st->st_atime = 0;
-	st->st_mtime = 0;
-
 	return 0;
 }
 
@@ -353,62 +365,97 @@ static int vfs_vnode_access(struct vnode *v, u32 mode)
 	return 0;
 }
 
+static void vfs_vnode_release(struct vnode *v)
+{
+	char *p;
+	char path[VFS_MAX_PATH];
+	struct mount *m;
+
+	if (!v) {
+		return;
+	}
+
+	m = v->v_mount;
+
+	if (m->m_root == v) {
+		vfs_vnode_vput(v);
+		return;
+	}
+
+	strncpy(path, v->v_path, sizeof(path));
+
+	vfs_vnode_vput(v);
+
+	while (1) {
+		p = strrchr(path, '/');
+		*p = '\0';
+
+		if (path[0] == '\0') {
+			break;
+		}
+
+		v = vfs_vnode_lookup(m, path);
+		if (!v) {
+			continue;
+		}
+
+		/* vput for previous lookup */
+		vfs_vnode_vput(v);
+
+		/* vput for previous acquire */
+		vfs_vnode_vput(v);
+	}
+
+	/* vput for mount point root */
+	vfs_vnode_vput(m->m_root);
+}
+
 static int vfs_vnode_acquire(const char *path, struct vnode **vp)
 {
 	char *p;
 	char node[VFS_MAX_PATH];
-	char name[VFS_MAX_PATH];
 	struct mount *m;
 	struct vnode *dv, *v;
-	int err, i;
+	int err, i, j;
 
 	/* convert a full path name to its mount point and
 	 * the local node in the file system.
 	 */
-	if(vfs_findroot(path, &m, &p)) {
+	if (vfs_findroot(path, &m, &p)) {
 		return VMM_ENOTAVAIL;
 	}
 
-	strncpy(node, "/", sizeof(node));
-	strncat(node, p, sizeof(node));
-	if ((v = vfs_vnode_lookup(m, node))) {
-		/* vnode is already active */
-		*vp = v;
-		return VMM_OK;
-	}
-
 	/* find target vnode, started from root directory.
 	 * this is done to attach the fs specific data to
 	 * the target vnode.
 	 */
-	if ((dv = m->m_root) == NULL) {
+	if (!m->m_root) {
 		return VMM_ENOSYS;
 	}
-
+	dv = v = m->m_root;
 	vfs_vnode_vref(dv);
-	node[0] = '\0';
 
-	while(*p != '\0') {
-		/* get lower directory or file name. */
-		while(*p == '/') {
+	i = 0;
+	while (*p != '\0') {
+		while (*p == '/') {
 			p++;
 		}
 
-		for(i = 0; i < VFS_MAX_PATH; i++) {
-			if(*p == '\0' || *p == '/') {
-				break;
-			}
-			name[i] = *p++;
+		node[i] = '/';
+		i++;
+		j = i;
+		while (*p != '\0' && *p != '/') {
+			node[i] = *p;
+			p++;
+			i++;
 		}
-		name[i] = '\0';
+		node[i] = '\0';	
 
 		/* get a vnode for the target. */
-		strncat(node, "/", sizeof(node));
-		strncat(node, name, sizeof(node));
 		v = vfs_vnode_lookup(m, node);
-		if(v == NULL) {
+		if (v == NULL) {
 			v = vfs_vnode_vget(m, node);
-			if(v == NULL) {
+			if (v == NULL) {
 				vfs_vnode_vput(dv);
 				return VMM_ENOMEM;
 			}
@@ -416,125 +463,167 @@ static int vfs_vnode_acquire(const char *path, struct vnode **vp)
 			/* find a vnode in this directory. */
 			vmm_mutex_lock(&v->v_lock);
 			vmm_mutex_lock(&dv->v_lock);
-			err = dv->v_mount->m_fs->lookup(dv, name, v);
+			err = dv->v_mount->m_fs->lookup(dv, &node[j], v);
 			vmm_mutex_unlock(&dv->v_lock);
 			vmm_mutex_unlock(&v->v_lock);
-			if(err || (*p == '/' && v->v_type != VDIR)) {
+			if (err || (*p == '/' && v->v_type != VDIR)) {
 				/* not found */
-				vfs_vnode_vput(v);
-				vfs_vnode_vput(dv);
+				vfs_vnode_release(v);
 				return err;
 			}
 		}
 
-		vfs_vnode_vput(dv);
 		dv = v;
-		while(*p != '\0' && *p != '/') {
-			p++;
-		}
 	}
 
 	*vp = v;
 
-	return 0;
+	return VMM_OK;
 }
 
-static void vfs_vnode_release(struct vnode *v)
+/* If a mount point is removed abruptly (probably due to
+ * unplugging of a pluggable block device) then we need to
+ * flush all file descriptors and vnodes under this mount point.
+ *
+ * Note: This is a recursive function and must be called with
+ * mount point list locked
+ */
+static void vfs_force_unmount(struct mount *m)
 {
-	char *p;
-	char path[VFS_MAX_PATH];
-	struct mount *m;
-	struct vnode *vt;
+	int i;
+	bool found;
+	struct dlist *l;
+	struct vnode *v;
+	struct mount *tm;
 
-	if (!v) {
-		return;
-	}
-
-	strncpy(path, v->v_path, sizeof(path));
-	m = v->v_mount;
-
-	vfs_vnode_vput(v);
-
-	while (path[0] != '\0') {
-		p = strrchr(path, '/');
-		*p = '\0';
-
-		vt = vfs_vnode_lookup(m, path);
-		if (!vt) {
-			continue;
+	/* First flush mount points having 
+	 * covered node under this mount point
+	 */
+	while (1) {
+		/* Find temp mount point */
+		found = FALSE;
+		list_for_each(l, &vfsc.mnt_list) {
+			tm = list_entry(l, struct mount, m_link);
+			if (tm->m_covered &&
+			    tm->m_covered->v_mount == m) {
+				found = TRUE;
+				break;
+			}
+		}
+		if (!found) {
+			break;
 		}
 
-		/* vput for previous lookup */
-		vfs_vnode_vput(vt);
-
-		/* vput for previous acquire */
-		vfs_vnode_vput(vt);
+		/* Flush temp mount point */
+		vfs_force_unmount(tm);
 	}
+
+	/* Remove mount point from mount point list */
+	list_del(&m->m_link);
+
+	/* Flush all file descriptors using vnode from this mount point */
+	vmm_mutex_lock(&vfsc.fd_bmap_lock);
+	for (i = 0; i < VFS_MAX_FD; i++) {
+		if (bitmap_isset(vfsc.fd_bmap, i) &&
+		    (vfsc.fd[i].f_vnode->v_mount == m)) {
+			vmm_mutex_lock(&vfsc.fd[i].f_lock);
+			vfsc.fd[i].f_flags = 0;
+			vfsc.fd[i].f_offset = 0;
+			vfsc.fd[i].f_vnode = NULL;
+			vmm_mutex_unlock(&vfsc.fd[i].f_lock);
+			bitmap_clear(vfsc.fd_bmap, i, 1);
+		}
+	}
+	vmm_mutex_unlock(&vfsc.fd_bmap_lock);
+
+	/* Flush all vnodes from this mount point */
+	for (i = 0; i < VFS_VNODE_HASH_SIZE; i++) {
+		vmm_mutex_lock(&vfsc.vnode_list_lock[i]);
+
+		while (1) {
+			found = FALSE;
+			list_for_each(l, &vfsc.vnode_list[i]) {
+				v = list_entry(l, struct vnode, v_link);
+				if (v->v_mount == m) {
+					found = TRUE;
+					break;
+				}
+			}
+			if (!found) {
+				break;
+			}
+
+			/* Remove vnode from hash list */
+			list_del(&v->v_link);
+
+			/* Deallocate fs specific data from this vnode */
+			vmm_mutex_lock(&v->v_mount->m_lock);
+			v->v_mount->m_fs->vput(v->v_mount, v);
+			vmm_mutex_unlock(&v->v_mount->m_lock);
+
+			/* Free vnode */
+			vmm_free(v);
+		}
+
+		vmm_mutex_unlock(&vfsc.vnode_list_lock[i]);
+	}
+
+	/* Call filesytem unmount */
+	vmm_mutex_lock(&m->m_lock);
+	m->m_fs->unmount(m);
+	vmm_mutex_unlock(&m->m_lock);
+
+	/* Release covering filesystem vnode */
+	if (m->m_covered) {
+		vfs_vnode_release(m->m_covered);
+	}
+
+	/* Free mount point */
+	vmm_free(m);
 }
 
-/** Compare two path strings and return matched length. */
-static int count_match(const char *path, char *mount_root)
-{
-	int len = 0;
-
-	while (*path && *mount_root) {
-		if ((*path++) != (*mount_root++))
-			break;
-		len++;
-	}
-
-	if (*mount_root != '\0') {
-		return 0;
-	}
-
-	if ((len == 1) && (*(path - 1) == '/')) {
-		return 1;
-	}
-
-	if ((*path == '\0') || (*path == '/')) {
-		return len;
-	}
-
-	return 0;
-}
-
-int vfs_findroot(const char *path, struct mount **mp, char **root)
+static int vfs_blockdev_notification(struct vmm_notifier_block *nb,
+				     unsigned long evt, void *data)
 {
+	bool found;
 	struct dlist *l;
-	struct mount *m, *tmp;
-	int len, max_len = 0;
+	struct mount *m;
+	struct vmm_blockdev_event *e = data;
 
-	if (!path || !mp || !root) {
-		return VMM_EFAIL;
+	if (evt != VMM_BLOCKDEV_EVENT_UNREGISTER) {
+		/* We are only interested in unregister events so,
+		 * don't care about this event.
+		 */
+		return NOTIFY_DONE;
 	}
 
-	/* find mount point from nearest path */
-	m = NULL;
-
+	/* Lock mount point list */
 	vmm_mutex_lock(&vfsc.mnt_list_lock);
 
+	/* Find mount point using block device */
+	found = FALSE;
 	list_for_each(l, &vfsc.mnt_list) {
-		tmp = list_entry(l, struct mount, m_link);
-		len = count_match(path, tmp->m_path);
-		if(len > max_len) {
-			max_len = len;
-			m = tmp;
+		m = list_entry(l, struct mount, m_link);
+		if (m->m_dev == e->bdev) {
+			found = TRUE;
+			break;
 		}
 	}
+	if (!found) {
+		/* Did not find suitable mount point so,
+		 * don't care about this event.
+		 */
+		vmm_mutex_unlock(&vfsc.mnt_list_lock);
+		return NOTIFY_DONE;
+	}
 
+	/* Force unmount */
+	vfs_force_unmount(m);
+
+	/* Unlock mount point list */
 	vmm_mutex_unlock(&vfsc.mnt_list_lock);
 
-	if(m == NULL) {
-		return VMM_EFAIL;
-	}
-
-	*root = (char *)(path + max_len);
-	if(**root == '/') {
-		(*root)++;
-	}
-	*mp = m;
-
-	return VMM_OK;
+	return NOTIFY_OK;
 }
 
 int vfs_mount(const char *dir, const char *fsname, const char *dev, u32 flags)
@@ -543,12 +632,13 @@ int vfs_mount(const char *dir, const char *fsname, const char *dev, u32 flags)
 	struct dlist *l;
 	struct vmm_blockdev *bdev;
 	struct filesystem *fs;
-	struct mount *m;
+	struct mount *m, *tm;
 	struct vnode *v, *v_covered;
 
 	BUG_ON(!vmm_scheduler_orphan_context());
 
-	if (!dir || *dir == '\0') {
+	/* sanity check */
+	if (!dir || *dir == '\0' || !(flags & MOUNT_MASK)) {
 		return VMM_EINVALID;
 	}
 
@@ -557,13 +647,19 @@ int vfs_mount(const char *dir, const char *fsname, const char *dev, u32 flags)
 		return VMM_EINVALID;
 	}
 
-	/* NULL can be specified as a dev. */
+	/* NULL cannot be specified as a dev. */
 	if (dev != NULL) {
 		if (!(bdev = vmm_blockdev_find(dev))) {
 			return VMM_EINVALID;
 		}
 	} else {
-		bdev = NULL;
+		return VMM_EINVALID;
+	}
+
+	/* For read-only devices mount as read-only */
+	if (bdev->flags & VMM_BLOCKDEV_RDONLY) {
+		flags &= ~MOUNT_RW;
+		flags |= MOUNT_RDONLY;
 	}
 
 	/* create vfs mount entry. */
@@ -598,7 +694,7 @@ int vfs_mount(const char *dir, const char *fsname, const char *dev, u32 flags)
 
 	/* create a root vnode for this file system. */
 	if (!(v = vfs_vnode_vget(m, "/"))) {
-		if(m->m_covered) {
+		if (m->m_covered) {
 			vfs_vnode_release(m->m_covered);
 		}
 		vmm_free(m);
@@ -606,9 +702,7 @@ int vfs_mount(const char *dir, const char *fsname, const char *dev, u32 flags)
 	}
 	v->v_type = VDIR;
 	v->v_flags = VROOT;
-	if (!S_ISDIR(v->v_mode) || (v->v_mode & (S_IRWXU | S_IRWXG | S_IRWXO))) {
-		v->v_mode = S_IFDIR | S_IRWXU | S_IRWXG | S_IRWXO;
-	}
+	v->v_mode = S_IFDIR | S_IRWXU | S_IRWXG | S_IRWXO;
 	m->m_root = v;
 
 	/* call a file system specific routine. */
@@ -617,14 +711,14 @@ int vfs_mount(const char *dir, const char *fsname, const char *dev, u32 flags)
 	vmm_mutex_unlock(&m->m_lock);
 	if (err != 0) {
 		vfs_vnode_release(m->m_root);
-		if(m->m_covered) {
+		if (m->m_covered) {
 			vfs_vnode_release(m->m_covered);
 		}
 		vmm_free(m);
 		return err;
 	}
 
-	if(m->m_flags & MOUNT_RDONLY) {
+	if (m->m_flags & MOUNT_RDONLY) {
 		m->m_root->v_mode &= ~(S_IWUSR|S_IWGRP|S_IWOTH);
 	}
 
@@ -632,15 +726,15 @@ int vfs_mount(const char *dir, const char *fsname, const char *dev, u32 flags)
 	vmm_mutex_lock(&vfsc.mnt_list_lock);
 
 	list_for_each(l, &vfsc.mnt_list) {
-		m = list_entry(l, struct mount, m_link);
-		if (!strcmp(m->m_path, dir) ||
-		    ((dev != NULL) && (m->m_dev == bdev))) {
+		tm = list_entry(l, struct mount, m_link);
+		if (!strcmp(tm->m_path, dir) ||
+		    ((dev != NULL) && (tm->m_dev == bdev))) {
 			vmm_mutex_unlock(&vfsc.mnt_list_lock);
 			vmm_mutex_lock(&m->m_lock);
 			m->m_fs->unmount(m);
 			vmm_mutex_unlock(&m->m_lock);
 			vfs_vnode_release(m->m_root);
-			if(m->m_covered) {
+			if (m->m_covered) {
 				vfs_vnode_release(m->m_covered);
 			}
 			vmm_free(m);
@@ -654,6 +748,7 @@ int vfs_mount(const char *dir, const char *fsname, const char *dev, u32 flags)
 
 	return VMM_OK;
 }
+VMM_EXPORT_SYMBOL(vfs_mount);
 
 int vfs_unmount(const char *path)
 {
@@ -676,39 +771,43 @@ int vfs_unmount(const char *path)
 	}
 
 	/* root fs can not be unmounted. */
-	if (!found || !m->m_covered) {
+	if (!found) {
 		vmm_mutex_unlock(&vfsc.mnt_list_lock);
 		return VMM_EINVALID;
 	}
 
+	/* mount point reference count should be 1 
+	 * otherwise it is busy.
+	 */
+	if (arch_atomic_read(&m->m_refcnt) > 1) {
+		vmm_mutex_unlock(&vfsc.mnt_list_lock);
+		return VMM_EBUSY;
+	}
+
 	/* remove mount point and break */
 	list_del(&m->m_link);
 
 	vmm_mutex_unlock(&vfsc.mnt_list_lock);
 
-	/* close file descriptors using any vnode 
-	 * belonging to this mount point;
-	 */
-	vfs_fd_unmount_flush(m);
-
-	/* release all vnodes */
-	vfs_vnode_unmount_flush(m);
-
-	/* call filesytem unmount */
+	/* call filesytem msync & filesystem unmount */
 	vmm_mutex_lock(&m->m_lock);
-	err = m->m_fs->unmount(m);
+	err = m->m_fs->msync(m);
+	m->m_fs->unmount(m);
 	vmm_mutex_unlock(&m->m_lock);
-	if(err != 0) {
-		return err;
-	}
+
+	/* releae mount point root */
+	vfs_vnode_release(m->m_root);
 
 	/* release covering filesystem vnode */
-	vfs_vnode_release(m->m_covered);
+	if (m->m_covered) {
+		vfs_vnode_release(m->m_covered);
+	}
 
 	vmm_free(m);
 
-	return VMM_OK;
+	return err;
 }
+VMM_EXPORT_SYMBOL(vfs_unmount);
 
 struct mount *vfs_mount_get(int index)
 {
@@ -744,6 +843,7 @@ struct mount *vfs_mount_get(int index)
 
 	return ret;
 }
+VMM_EXPORT_SYMBOL(vfs_mount_get);
 
 u32 vfs_mount_count(void)
 {
@@ -762,6 +862,7 @@ u32 vfs_mount_count(void)
 
 	return retval;
 }
+VMM_EXPORT_SYMBOL(vfs_mount_count);
 
 static int vfs_lookup_dir(const char *path, struct vnode **vp, char **name)
 {
@@ -890,21 +991,11 @@ int vfs_open(const char *path, u32 flags, u32 mode)
 	f->f_flags = flags;
 	f->f_offset = 0;
 
-	/* request to file system */
-	vmm_mutex_lock(&v->v_lock);
-	err = v->v_mount->m_fs->open(v, f);
-	vmm_mutex_unlock(&v->v_lock);
-	if (err) {
-		vmm_mutex_unlock(&f->f_lock);
-		vfs_fd_free(fd);
-		vfs_vnode_release(v);
-		return err;
-	}
-
 	vmm_mutex_unlock(&f->f_lock);
 
 	return fd;
 }
+VMM_EXPORT_SYMBOL(vfs_open);
 
 int vfs_close(int fd)
 {
@@ -928,7 +1019,7 @@ int vfs_close(int fd)
 	}
 
 	vmm_mutex_lock(&v->v_lock);
-	err = v->v_mount->m_fs->close(v, f);
+	err = v->v_mount->m_fs->sync(v);
 	vmm_mutex_unlock(&v->v_lock);
 	if (err) {
 		vmm_mutex_unlock(&f->f_lock);
@@ -943,6 +1034,7 @@ int vfs_close(int fd)
 
 	return VMM_OK;
 }
+VMM_EXPORT_SYMBOL(vfs_close);
 
 size_t vfs_read(int fd, void *buf, size_t len)
 {
@@ -968,6 +1060,10 @@ size_t vfs_read(int fd, void *buf, size_t len)
 		vmm_mutex_unlock(&f->f_lock);
 		return 0;
 	}
+	if (v->v_type != VREG) {
+		vmm_mutex_unlock(&f->f_lock);
+		return 0;
+	}
 
 	if (!(f->f_flags & O_RDONLY)) {
 		vmm_mutex_unlock(&f->f_lock);
@@ -975,7 +1071,7 @@ size_t vfs_read(int fd, void *buf, size_t len)
 	}
 
 	vmm_mutex_lock(&v->v_lock);
-	ret = v->v_mount->m_fs->read(v, f, buf, len);
+	ret = v->v_mount->m_fs->read(v, f->f_offset, buf, len);
 	vmm_mutex_unlock(&v->v_lock);
 
 	f->f_offset += ret;
@@ -984,6 +1080,7 @@ size_t vfs_read(int fd, void *buf, size_t len)
 
 	return ret;
 }
+VMM_EXPORT_SYMBOL(vfs_read);
 
 size_t vfs_write(int fd, void *buf, size_t len)
 {
@@ -1009,6 +1106,10 @@ size_t vfs_write(int fd, void *buf, size_t len)
 		vmm_mutex_unlock(&f->f_lock);
 		return 0;
 	}
+	if (v->v_type != VREG) {
+		vmm_mutex_unlock(&f->f_lock);
+		return 0;
+	}
 
 	if (!(f->f_flags & O_WRONLY)) {
 		vmm_mutex_unlock(&f->f_lock);
@@ -1016,7 +1117,7 @@ size_t vfs_write(int fd, void *buf, size_t len)
 	}
 
 	vmm_mutex_lock(&v->v_lock);
-	ret = v->v_mount->m_fs->write(v, f, buf, len);
+	ret = v->v_mount->m_fs->write(v, f->f_offset, buf, len);
 	vmm_mutex_unlock(&v->v_lock);
 
 	f->f_offset += ret;
@@ -1025,6 +1126,7 @@ size_t vfs_write(int fd, void *buf, size_t len)
 
 	return ret;
 }
+VMM_EXPORT_SYMBOL(vfs_write);
 
 loff_t vfs_lseek(int fd, loff_t off, int whence)
 {
@@ -1086,7 +1188,7 @@ loff_t vfs_lseek(int fd, loff_t off, int whence)
 		return ret;
 	}
 
-	if (v->v_mount->m_fs->seek(v, f, off)) {
+	if (off <= (loff_t)(v->v_size)) {
 		f->f_offset = off;
 	}
 
@@ -1098,6 +1200,7 @@ loff_t vfs_lseek(int fd, loff_t off, int whence)
 
 	return ret;
 }
+VMM_EXPORT_SYMBOL(vfs_lseek);
 
 int vfs_fsync(int fd)
 {
@@ -1126,13 +1229,47 @@ int vfs_fsync(int fd)
 	}
 
 	vmm_mutex_lock(&v->v_lock);
-	err = v->v_mount->m_fs->fsync(v, f);
+	err = v->v_mount->m_fs->sync(v);
 	vmm_mutex_unlock(&v->v_lock);
 
 	vmm_mutex_unlock(&f->f_lock);
 
 	return err;
 }
+VMM_EXPORT_SYMBOL(vfs_fsync);
+
+int vfs_fchmod(int fd, u32 mode)
+{
+	int err;
+	struct vnode *v;
+	struct file *f;
+
+	BUG_ON(!vmm_scheduler_orphan_context());
+
+	f = vfs_fd_to_file(fd);
+	if (!f) {
+		return VMM_EINVALID;
+	}
+
+	vmm_mutex_lock(&f->f_lock);
+
+	v = f->f_vnode;
+	if (!v) {
+		vmm_mutex_unlock(&f->f_lock);
+		return VMM_EINVALID;
+	}
+
+	mode &= (S_IRWXU|S_IRWXG|S_IRWXO);
+
+	vmm_mutex_lock(&v->v_lock);
+	err = v->v_mount->m_fs->chmod(v, mode);
+	vmm_mutex_unlock(&v->v_lock);
+
+	vmm_mutex_unlock(&f->f_lock);
+
+	return err;
+}
+VMM_EXPORT_SYMBOL(vfs_fchmod);
 
 int vfs_fstat(int fd, struct stat *st)
 {
@@ -1165,6 +1302,7 @@ int vfs_fstat(int fd, struct stat *st)
 
 	return err;
 }
+VMM_EXPORT_SYMBOL(vfs_fstat);
 
 int vfs_opendir(const char *name)
 {
@@ -1195,7 +1333,7 @@ int vfs_opendir(const char *name)
 		return VMM_EINVALID;
 	}
 
-	if(v->v_type != VDIR) {
+	if (v->v_type != VDIR) {
 		vmm_mutex_unlock(&f->f_lock);
 		vfs_close(fd);
 		return VMM_EINVALID;
@@ -1205,6 +1343,7 @@ int vfs_opendir(const char *name)
 
 	return fd;
 }
+VMM_EXPORT_SYMBOL(vfs_opendir);
 
 int vfs_closedir(int fd)
 {
@@ -1226,7 +1365,7 @@ int vfs_closedir(int fd)
 		return VMM_EINVALID;
 	}
 
-	if(v->v_type != VDIR) {
+	if (v->v_type != VDIR) {
 		vmm_mutex_unlock(&f->f_lock);
 		return VMM_EINVALID;
 	}
@@ -1235,6 +1374,7 @@ int vfs_closedir(int fd)
 
 	return vfs_close(fd);
 }
+VMM_EXPORT_SYMBOL(vfs_closedir);
 
 int vfs_readdir(int fd, struct dirent *dir)
 {
@@ -1267,7 +1407,7 @@ int vfs_readdir(int fd, struct dirent *dir)
 	}
 
 	vmm_mutex_lock(&v->v_lock);
-	err = v->v_mount->m_fs->readdir(v, f, dir);
+	err = v->v_mount->m_fs->readdir(v, f->f_offset, dir);
 	vmm_mutex_unlock(&v->v_lock);
 	if (!err) {
 		f->f_offset += dir->d_reclen;
@@ -1277,6 +1417,7 @@ int vfs_readdir(int fd, struct dirent *dir)
 
 	return err;
 }
+VMM_EXPORT_SYMBOL(vfs_readdir);
 
 int vfs_rewinddir(int fd)
 {
@@ -1309,6 +1450,7 @@ int vfs_rewinddir(int fd)
 
 	return VMM_OK;
 }
+VMM_EXPORT_SYMBOL(vfs_rewinddir);
 
 int vfs_mkdir(const char *path, u32 mode)
 {
@@ -1342,13 +1484,22 @@ int vfs_mkdir(const char *path, u32 mode)
 	mode |= S_IFDIR;
 
 	vmm_mutex_lock(&dv->v_lock);
+
 	err = dv->v_mount->m_fs->mkdir(dv, name, mode);
+	if (err) {
+		goto fail;
+	}
+
+	err = dv->v_mount->m_fs->sync(dv);
+
+fail:
 	vmm_mutex_unlock(&dv->v_lock);
 
 	vfs_vnode_release(dv);
 
 	return err;
 }
+VMM_EXPORT_SYMBOL(vfs_mkdir);
 
 static int vfs_check_dir_empty(const char *path)
 {
@@ -1356,7 +1507,7 @@ static int vfs_check_dir_empty(const char *path)
 	struct dirent dir;
 
 	if ((fd = vfs_opendir(path)) < 0) {
-		return VMM_EINVALID;
+		return fd;
 	}
 
 	count = 0;
@@ -1365,7 +1516,8 @@ static int vfs_check_dir_empty(const char *path)
 		if (err) {
 			break;
 		}
-		if (!strcmp(dir.d_name, ".") || !strcmp(dir.d_name, "..")) {
+		if ((strcmp(dir.d_name, ".") != 0) && 
+		    (strcmp(dir.d_name, "..") != 0)) {
 			count++;
 		}
 		if (count) {
@@ -1420,7 +1572,20 @@ int vfs_rmdir(const char *path)
 
 	vmm_mutex_lock(&dv->v_lock);
 	vmm_mutex_lock(&v->v_lock);
+
 	err = dv->v_mount->m_fs->rmdir(dv, v, name);
+	if (err) {
+		goto fail;
+	}
+
+	err = v->v_mount->m_fs->sync(v);
+	if (err) {
+		goto fail;
+	}
+
+	err = dv->v_mount->m_fs->sync(dv);
+
+fail:	
 	vmm_mutex_unlock(&v->v_lock);
 	vmm_mutex_unlock(&dv->v_lock);
 
@@ -1429,114 +1594,115 @@ int vfs_rmdir(const char *path)
 
 	return err;
 }
+VMM_EXPORT_SYMBOL(vfs_rmdir);
 
-int vfs_rename(char *src, char *dest)
+int vfs_rename(const char *src, const char *dest)
 {
 	int err, len;
 	char *sname, *dname;
-	struct vnode *v1, *v2 = NULL, *dv1, *dv2;
+	struct vnode *v1, *v2, *sv, *dv;
 
 	BUG_ON(!vmm_scheduler_orphan_context());
 
+	/* if source and dest are the same, do nothing */
+	if (!strncmp(src, dest, VFS_MAX_PATH)) {
+		return VMM_EINVALID;
+	}
+
+	/* if source is root directory then, do nothing */
+	if (!strcmp(src, "/")) {
+		return VMM_EINVALID;
+	}
+
+	/* if dest is a directory of source then, do nothing */
+	len = strlen(src);
+	if ((len < strlen(dest)) && 
+	    !strncmp(src, dest, len) &&
+	    (dest[len] == '/')) {
+		return VMM_EINVALID;
+	}
+
+	/* get source v1 */
 	if ((err = vfs_vnode_acquire(src, &v1))) {
 		return err;
 	}
+
+	/* check source permission */
 	if ((err = vfs_vnode_access(v1, W_OK))) {
-		goto err1;
+		goto fail1;
 	}
 
-	/* if source and dest are the same, do nothing */
-	if (!strncmp(src, dest, VFS_MAX_PATH)) {
-		goto err1;
-	}
-
-	/* check if target is directory of source */
-	len = strlen(dest);
-	if (!strncmp(src, dest, len)) {
-		err = VMM_EINVALID;
-		goto err1;
-	}
-
-	/* is the source busy ? */
+	/* check if source is busy ? */
 	if (arch_atomic_read(&v1->v_refcnt) >= 2) {
 		err = VMM_EBUSY;
-		goto err1;
+		goto fail1;
 	}
 
-	/* check type of source & target */
+	/* get sv and sname */
+	if ((err = vfs_lookup_dir(src, &sv, &sname))) {
+		goto fail1;
+	}
+
+	/* check if dest exists */
 	err = vfs_vnode_acquire(dest, &v2);
 	if (!err) {
-		/* target exists */
-		if (v1->v_type == VDIR && v2->v_type != VDIR) {
-			err = VMM_EINVALID;
-			goto err2;
-		} else if (v1->v_type != VDIR && v2->v_type == VDIR) {
-			err = VMM_EINVALID;
-			goto err2;
-		}
-		if (v2->v_type == VDIR && vfs_check_dir_empty(dest)) {
-			err = VMM_EINVALID;
-			goto err2;
-		}
-		if (arch_atomic_read(&v2->v_refcnt) >= 2) {
-			err = VMM_EBUSY;
-			goto err2;
-		}
+		vfs_vnode_release(v2);
+		err = VMM_EEXIST;
+		goto fail2;
 	}
 
-	dname = (char *)strrchr(dest, '/');
-	if (!dname) {
-		err = VMM_EINVALID;
-		goto err2;
-	}
-	if (dname == dest) {
-		dest = "/";
+	/* get dv and dname */
+	if ((err = vfs_lookup_dir(dest, &dv, &dname))) {
+		goto fail2;
 	}
 
-	*dname = 0;
-	dname++;
-
-	if ((err = vfs_lookup_dir(src, &dv1, &sname))) {
-		goto err2;
-	}
-
-	if ((err = vfs_vnode_acquire(dest, &dv2)) != 0) {
-		goto err3;
-	}
-
-	/* the source and dest must be same file system */
-	if (dv1->v_mount != dv2->v_mount) {
+	/* the sv and dv must be on same file system */
+	if (sv->v_mount != dv->v_mount) {
 		err = VMM_EIO;
-		goto err4;
+		goto fail3;
 	}
 
 	vmm_mutex_lock(&v1->v_lock);
-	if (v2) {
-		vmm_mutex_lock(&v2->v_lock);
+
+	vmm_mutex_lock(&sv->v_lock);
+
+	if (dv != sv) {
+		vmm_mutex_lock(&dv->v_lock);
 	}
-	vmm_mutex_lock(&dv1->v_lock);
-	vmm_mutex_lock(&dv2->v_lock);
-	err = dv1->v_mount->m_fs->rename(dv1, v1, sname, dv2, v2, dname);
-	vmm_mutex_unlock(&dv2->v_lock);
-	vmm_mutex_unlock(&dv1->v_lock);
-	if (v2) {
-		vmm_mutex_unlock(&v2->v_lock);
+
+	err = sv->v_mount->m_fs->rename(sv, sname, v1, dv, dname);
+	if (err) {
+		goto fail4;
 	}
+
+	err = sv->v_mount->m_fs->sync(sv);
+	if (err) {
+		goto fail4;
+	}
+
+	if (dv != sv) {
+		err = dv->v_mount->m_fs->sync(dv);
+	}
+
+fail4:
+	if (dv != sv) {
+		vmm_mutex_unlock(&dv->v_lock);
+	}
+
+	vmm_mutex_unlock(&sv->v_lock);
+
 	vmm_mutex_unlock(&v1->v_lock);
 
- err4:
-	vfs_vnode_release(dv2);
- err3:
-	vfs_vnode_release(dv1);
- err2:
-	if (v2) {
-		vfs_vnode_release(v2);
-	}
- err1:
+fail3:
+	vfs_vnode_release(dv);
+fail2:
+	vfs_vnode_release(sv);
+fail1:
 	vfs_vnode_release(v1);
 
 	return err;
 }
+VMM_EXPORT_SYMBOL(vfs_rename);
 
 int vfs_unlink(const char *path)
 {
@@ -1575,17 +1741,39 @@ int vfs_unlink(const char *path)
 		return err;
 	}
 
-	vmm_mutex_lock(&dv->v_lock);
 	vmm_mutex_lock(&v->v_lock);
+
+	err = v->v_mount->m_fs->truncate(v, 0);
+	if (err) {
+		goto fail1;
+	}
+
+	err = v->v_mount->m_fs->sync(v);
+	if (err) {
+		goto fail1;
+	}
+
+	vmm_mutex_lock(&dv->v_lock);
+
 	err = dv->v_mount->m_fs->remove(dv, v, name);
-	vmm_mutex_unlock(&v->v_lock);
+	if (err) {
+		goto fail2;
+	}
+
+	err = dv->v_mount->m_fs->sync(dv);
+
+fail2:
 	vmm_mutex_unlock(&dv->v_lock);
 
-	vfs_vnode_release(v);
+fail1:
+	vmm_mutex_unlock(&v->v_lock);
+
 	vfs_vnode_release(dv);
+	vfs_vnode_release(v);
 
 	return err;
 }
+VMM_EXPORT_SYMBOL(vfs_unlink);
 
 int vfs_access(const char *path, u32 mode)
 {
@@ -1608,6 +1796,42 @@ int vfs_access(const char *path, u32 mode)
 
 	return err;
 }
+VMM_EXPORT_SYMBOL(vfs_access);
+
+int vfs_chmod(const char *path, u32 mode)
+{
+	int err;
+	struct vnode *v;
+
+	BUG_ON(!vmm_scheduler_orphan_context());
+
+	if (!path) {
+		return VMM_EINVALID;
+	}
+
+	if ((err = vfs_vnode_acquire(path, &v))) {
+		return err;
+	}
+
+	mode &= (S_IRWXU|S_IRWXG|S_IRWXO);
+
+	vmm_mutex_lock(&v->v_lock);
+
+	err = v->v_mount->m_fs->chmod(v, mode);
+	if (err) {
+		goto fail;
+	}
+
+	err = v->v_mount->m_fs->sync(v);
+
+fail:
+	vmm_mutex_unlock(&v->v_lock);
+
+	vfs_vnode_release(v);
+
+	return err;
+}
+VMM_EXPORT_SYMBOL(vfs_chmod);
 
 int vfs_stat(const char *path, struct stat *st)
 {
@@ -1630,6 +1854,7 @@ int vfs_stat(const char *path, struct stat *st)
 
 	return err;
 }
+VMM_EXPORT_SYMBOL(vfs_stat);
 
 int vfs_filesystem_register(struct filesystem *fs)
 {
@@ -1668,6 +1893,7 @@ int vfs_filesystem_register(struct filesystem *fs)
 
 	return VMM_OK;
 }
+VMM_EXPORT_SYMBOL(vfs_filesystem_register);
 
 int vfs_filesystem_unregister(struct filesystem *fs)
 {
@@ -1709,6 +1935,7 @@ int vfs_filesystem_unregister(struct filesystem *fs)
 
 	return VMM_OK;
 }
+VMM_EXPORT_SYMBOL(vfs_filesystem_unregister);
 
 struct filesystem *vfs_filesystem_find(const char *name)
 {
@@ -1743,6 +1970,7 @@ struct filesystem *vfs_filesystem_find(const char *name)
 
 	return ret;
 }
+VMM_EXPORT_SYMBOL(vfs_filesystem_find);
 
 struct filesystem *vfs_filesystem_get(int index)
 {
@@ -1778,6 +2006,7 @@ struct filesystem *vfs_filesystem_get(int index)
 
 	return ret;
 }
+VMM_EXPORT_SYMBOL(vfs_filesystem_get);
 
 u32 vfs_filesystem_count(void)
 {
@@ -1796,6 +2025,7 @@ u32 vfs_filesystem_count(void)
 
 	return retval;
 }
+VMM_EXPORT_SYMBOL(vfs_filesystem_count);
 
 static int __init vfs_init(void)
 {
@@ -1825,12 +2055,17 @@ static int __init vfs_init(void)
 		INIT_MUTEX(&vfsc.fd[i].f_lock);
 	}
 
+	vfsc.bdev_client.notifier_call = &vfs_blockdev_notification;
+	vfsc.bdev_client.priority = 0;
+	vmm_blockdev_register_client(&vfsc.bdev_client);
+
 	return VMM_OK;
 }
 
 static void __exit vfs_exit(void)
 {
-	/* For now nothing to be done for exit. */
+	vmm_blockdev_unregister_client(&vfsc.bdev_client);
+	vmm_free(vfsc.fd_bmap);
 }
 
 VMM_DECLARE_MODULE(MODULE_DESC,
